//
//  GPSController.m
//  iCMED
//
//  Created by Jose Mariano Garcia Luengo on 01/03/13.
//  Copyright (c) 2013 Jose Mariano Garcia Luengo. All rights reserved.
//
#import "GPSController.h"
#import "MedicosController.h"
#import "Comun.h"

@interface GPSController ()

@end

@implementation GPSController

- (void)viewDidLoad {
    @try {
        [self setPosition:@"0.000000,0.000000"];
        
        // Create CLLocationMamager if not exists yet.
        if (nil == [self locationManager])
            [self setLocationManager:[[CLLocationManager alloc] init]];
        
        [self setPosition:[NSString stringWithFormat:@"%f,%f", [self locationManager].location.coordinate.latitude, [self locationManager].location.coordinate.longitude]];
        
        if (!([[self position] isEqualToString:@"0.000000,0.000000"]) && [self walkingButton].hidden == YES) {
            [self walkingButton].hidden = NO;
            [self drivingButton].hidden = NO;
        }
        
        // Create CLGeocoder if not exists yet.
        if (![self geocoder]) {
            [self setGeocoder:[[CLGeocoder alloc] init]];
        }
        
        // Create MKPointAnnotation at the choosen medical service address.
        [[self geocoder] geocodeAddressString:[self destination] completionHandler:^(NSArray *placemarks, NSError *error) {
            if ([placemarks count] > 0) {
                CLPlacemark *placemark = [placemarks objectAtIndex:0];
                CLLocation *location = placemark.location;
                CLLocationCoordinate2D coordinate = location.coordinate;
                [self setCoordinates:[NSString stringWithFormat:@"%f, %f", coordinate.latitude, coordinate.longitude]];
                if ([placemark.areasOfInterest count] > 0) {
                    NSString *areaOfInterest = [placemark.areasOfInterest objectAtIndex:0];
                    [self setName:areaOfInterest];
                }
                
                CLLocationCoordinate2D Medico;
                Medico.latitude = coordinate.latitude;
                Medico.longitude = coordinate.longitude;
                
                MKPointAnnotation *aPoint = [[MKPointAnnotation alloc] init];
                aPoint.coordinate = Medico;
                aPoint.title = [NSString stringWithString: [self fullName]];
                aPoint.subtitle = [NSString stringWithFormat:@"%@ en %@", [self especDescription], [self address]];
                
                [[self map] addAnnotation:aPoint];
                
                MKCoordinateRegion viewRegion = MKCoordinateRegionMakeWithDistance(Medico, 400, 400);
                MKCoordinateRegion adjustedRegion = [[self map] regionThatFits:viewRegion];
                [[self map] setRegion:adjustedRegion animated:YES];
            }
        }];
        
        [super viewDidLoad];        
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.viewDidLoad: %@", e);
    }
}

- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray *)locations
{
    @try {
        CLLocation *newPosition = [locations lastObject];
        
        [self setLat:newPosition.coordinate.latitude];
        [self setLon:newPosition.coordinate.longitude];
        
        [self setPosition:[NSString stringWithFormat:@"%f,%f", [self lat], [self lon]]];
        
        if (![[self position] isEqualToString:@"0.000000,0.000000"] && [self walkingButton].hidden == YES) {
            [self walkingButton].hidden = NO;
            [self drivingButton].hidden = NO;
        }
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.didUpdateLocations: %@", e);
    }
}

// Personalize Annotation with a red cross pin image.
- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:(id <MKAnnotation>)annotation {
    @try {
        static NSString *identifier = @"MyLocation";
        
        if ([annotation isKindOfClass:[MKPointAnnotation class]]) {
            MKAnnotationView *annotationView = (MKAnnotationView *) [mapView dequeueReusableAnnotationViewWithIdentifier:identifier];
            if (annotationView == nil) {
                annotationView = [[MKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:identifier];
                annotationView.enabled = YES;
                annotationView.canShowCallout = YES;
                annotationView.image = [UIImage imageNamed:@"CRUZ-ROJA 3D pin.png"];
            } else {
                annotationView.annotation = annotation;
            }
            
            return annotationView;
        }
        
        return nil;
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.viewForAnnotation: %@", e);
    }
}

- (IBAction)walkingButtonClick:(id)sender {
    @try {
        if (!self.walkingRouteLine) {
            [self setUrl:[NSString stringWithFormat:@"https://maps.googleapis.com/maps/api/directions/json?origin=%@&destination=%@&mode=walking&sensor=false", [self position], [self destination]]];
            
            [self howToGo:[self url]];
            
            [self setWalkingRouteLine:[self routeLine]];
            [self setWalkingRouteRect:[self routeRect]];
        }
        
        if([[[self map] overlays] count] > 0)
            [[self map] removeOverlays:[[self map] overlays]];
        
        [self setRouteLineView:nil];
        [self setRouteLine:[self walkingRouteLine]];
        
        [[self map] addOverlay:[self walkingRouteLine]];
        [[self map] setVisibleMapRect:[self walkingRouteRect]];
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.walkingButtonClick: %@", e);
    }
}

- (IBAction)drivingButtonClick:(id)sender {
    @try {
        if(!self.drivingRouteLine) {
            [self setUrl:[NSString stringWithFormat:@"https://maps.googleapis.com/maps/api/directions/json?origin=%@&destination=%@&sensor=false", [self position], [self destination]]];
            
            [self howToGo:[self url]];
            
            [self setDrivingRouteLine:[self routeLine]];
            [self setDrivingRouteRect:[self routeRect]];
        }
        
        if([[[self map] overlays] count] > 0)
            [[self map] removeOverlays:[[self map] overlays]];
        
        [self setRouteLineView:nil];
        [self setRouteLine:[self drivingRouteLine]];
        
        [[self map] addOverlay:[self drivingRouteLine]];
        [[self map] setVisibleMapRect:[self drivingRouteRect]];
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.drivingButtonClick: %@", e);
    }
}

- (void) howToGo:(NSString *) urlDrivingOrWalking {
    @try {
        urlDrivingOrWalking = [urlDrivingOrWalking stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
        //url = [NSURL URLWithString:urlDrivingOrWalking];
        
        [self setUrlData:[NSData dataWithContentsOfURL:[NSURL URLWithString:urlDrivingOrWalking]]];
        
        if ([self urlData].length == 0) {
            [Comun showAlert:@"No se han recibido datos."];
        } else {
            NSLog(@"Succeeded! Received %d bytes of data",[_urlData length]);
            
            NSString *decodingPoints = [self encodedPointsFromGoogleapisData];
            
            NSMutableArray *decoding = [self decodePolyline:decodingPoints];
            
            // Creamos el overlay
            [self loadRouteFromWaypoints:decoding];
        }
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.howToGo: %@", e);
    }
}

- (NSString *)encodedPointsFromGoogleapisData {
    @try {
        NSError *error = nil;
        NSMutableDictionary *jsonArray = [NSJSONSerialization JSONObjectWithData:_urlData options:NSJSONReadingMutableContainers error: &error];
        
        if (!jsonArray) {
            NSLog(@"Error parsing JSON: %@", error);
            NSLog(@"_receivedData: %@", [_urlData description]);
        } else {
            if (![[jsonArray valueForKey:@"status"] isEqualToString:@"ZERO_RESULTS"]) {
                NSDictionary *routes = [[jsonArray valueForKey:@"routes"] objectAtIndex:0];
                NSDictionary *overviewPolyline = [routes valueForKey:@"overview_polyline"];
                return [overviewPolyline valueForKey:@"points"];
            }
        }
        
        return nil;
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.encodedPointsFromGoogleapisData: %@", e);
    }
}

- (NSMutableArray *) decodePolyline:(NSString *)encodedPoints {
    @try {
        NSString *escapedEncodedPoints = [encodedPoints stringByReplacingOccurrencesOfString:@"\\\\" withString:@"\\"];
        int len = [escapedEncodedPoints length];
        NSMutableArray *waypoints = [[NSMutableArray alloc] init];
        int index = 0;
        float lat = 0;
        float lng = 0;
        
        while (index < len) {
            char b;
            int shift = 0;
            int result = 0;
            do {
                b = [escapedEncodedPoints characterAtIndex:index++] - 63;
                result |= (b & 0x1f) << shift;
                shift += 5;
            } while (b >= 0x20);
            
            float dlat = ((result & 1) ? ~(result >> 1) : (result >> 1));
            lat += dlat;
            
            shift = 0;
            result = 0;
            do {
                b = [escapedEncodedPoints characterAtIndex:index++] - 63;
                result |= (b & 0x1f) << shift;
                shift += 5;
            } while (b >= 0x20);
            
            float dlng = ((result & 1) ? ~(result >> 1) : (result >> 1));
            lng += dlng;
            
            NSNumber *latitude = [[NSNumber alloc] initWithFloat:lat * 1e-5];
            NSNumber *longitude = [[NSNumber alloc] initWithFloat:lng * 1e-5];
            CLLocation *loc = [[CLLocation alloc] initWithLatitude:[latitude floatValue] longitude:[longitude floatValue]];
            [waypoints addObject:loc];
        }
        return waypoints;
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.decodePolyline: %@", e);
    }
}

-(void)loadRouteFromWaypoints:(NSArray *)waypoints
{
    @try {
        // while we create the route points, we will also be calculating the bounding box of our route
        // so we can easily zoom in on it.
        MKMapPoint northEastPoint;
        MKMapPoint southWestPoint;
        
        // create a c array of points.
        MKMapPoint* pointArr = malloc(sizeof(CLLocationCoordinate2D) * waypoints.count);
        
        for(int idx = 0; idx < waypoints.count; idx++)
        {
            //
            CLLocation *location = [waypoints objectAtIndex:idx];
            
            // create our coordinate and add it to the correct spot in the array
            CLLocationCoordinate2D coordinate = location.coordinate;
            
            MKMapPoint point = MKMapPointForCoordinate(coordinate);
            
            //
            // adjust the bounding box
            //
            
            // if it is the first point, just use them, since we have nothing to compare to yet.
            if (idx == 0) {
                northEastPoint = point;
                southWestPoint = point;
            }
            else
            {
                if (point.x > northEastPoint.x)
                    northEastPoint.x = point.x;
                if(point.y > northEastPoint.y)
                    northEastPoint.y = point.y;
                if (point.x < southWestPoint.x)
                    southWestPoint.x = point.x;
                if (point.y < southWestPoint.y)
                    southWestPoint.y = point.y;
            }
            
            pointArr[idx] = point;
            
        }
        
        self.routeLine = nil;
        
        // create the polyline based on the array of points.
        self.routeLine = [MKPolyline polylineWithPoints:pointArr count:waypoints.count];
        
        _routeRect = MKMapRectMake(southWestPoint.x, southWestPoint.y, northEastPoint.x - southWestPoint.x, northEastPoint.y - southWestPoint.y);
        
        // free used memory
        free(pointArr);
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.loadRouteFromWaypoints: %@", e);
    }
}

- (MKOverlayView *)mapView:(MKMapView *)mapView viewForOverlay:(id )overlay
{
    @try {
        MKOverlayView* overlayView = nil;
        
        if(overlay == self.routeLine) {
            // We create an Overlay if not exists yet..
            if(nil == self.routeLineView)
            {
                // Atributos de la linea que traza la ruta, como color y grosor de la misma.
                self.routeLineView = [[MKPolylineView alloc] initWithPolyline:self.routeLine];
                self.routeLineView.fillColor = [UIColor colorWithRed:0.0f green:1.0f blue:0.0f alpha:0.5f];
                self.routeLineView.strokeColor = [UIColor colorWithRed:0.0f green:1.0f blue:0.0f alpha:0.5f];
                self.routeLineView.lineWidth = 7;
            }
            
            overlayView = self.routeLineView;
        }
        
        return overlayView;
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.viewForOverlay: %@", e);
    }
}

- (void)alertView:(UIAlertView *)alerta clickedButtonAtIndex:(NSInteger)buttonIndex
{
    @try {
        NSString *title = [alerta buttonTitleAtIndex:buttonIndex];
        
        if([title isEqualToString:@"SI"])
        {
            [self performSegueWithIdentifier:[NSString stringWithFormat:@"%s", "CerrarSesion"] sender:self];
        }
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.alertView.clickedButtonAtIndex: %@", e);
    }
}

- (IBAction)exitButtonClick:(id)sender {
    @try {
        UIAlertView *alert = [[UIAlertView alloc]
                              initWithTitle:[NSString stringWithFormat:@"Atención:"]
                              message:[NSString stringWithFormat:@"¿Seguro que quiere abandonar la sesión?."]
                              delegate:self
                              cancelButtonTitle:@"SI"
                              otherButtonTitles:@"NO", nil];
        [alert show];
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.exitButtonClick: %@", e);
    }
}

-(void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender{
    @try {
        if ([[segue identifier] isEqualToString:@"BackFromGPS"]) {
            MedicosController *backView = [segue destinationViewController];
            
            [backView setState:[self state]];
            [backView setCity:[self city]];
            [backView setEspec:[self espec]];
            [backView setUserCode:[self userCode]];
            [backView setUserType:[self userType]];
        }
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en GPSController.prepareForSegue: %@", e);
    }
}

@end
