//
//  MedicosController.m
//  iCMED
//
//  Created by Jose Mariano Garcia Luengo on 01/03/13.
//  Copyright (c) 2013 Jose Mariano Garcia Luengo. All rights reserved.
//
#import "MedicosController.h"
#import "BuscarMedicoController.h"
#import "GPSController.h"
#import "Comun.h"

@interface MedicosController ()

@end

@implementation MedicosController


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
    }
    return self;
}

- (void)viewDidLoad
{
    @try {
        [self setWeb:[NSString stringWithFormat:@"%@%s%d%s%d%s%d", WEB_TFC, "medicos&provincia=", [self state], "&poblacion=", [self city], "&especialidad=", [self espec]]];
        [self setUrl:[NSURL URLWithString:[self web]]];
        
        [self setUrlData:[NSData dataWithContentsOfURL:[self url]]];
        
        [self setError:nil];
        
        [self setDict:[NSJSONSerialization JSONObjectWithData:[self urlData] options:kNilOptions error:[self error]]];
        
        if ([self error] != nil) {
            // NSLog(@"ERROR: %@", [[self error] localizedDescription]);
        } else {
            UIFont *fuente = [UIFont systemFontOfSize:15];
            
            NSDictionary *atributos = @{
                                        UITextAttributeFont: fuente
                                        };
            
            [[self navBarTitle] setTitleTextAttributes:atributos];
            
            [self setRows:[[self dict] objectForKey:@"datos"]];
            [self setDict:[[self rows] objectAtIndex:0]];
            [[[self navBarTitle] topItem] setTitle:[NSString stringWithFormat:@"%@ en %@", [[self dict] objectForKey:@"especialidad"], [[self dict] objectForKey:@"poblacion"]]];
        }
        
        [super viewDidLoad];
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en MedicosController.viewDidLoad: %@", e);
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [[self rows] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    @try {
        static NSString *CellIdentifier = @"Cell";
        UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
        
        if (cell == nil) {
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
        }
        // Configure the cell...
        [self setDict:[[self rows] objectAtIndex:indexPath.row]];
        
        cell.textLabel.text = [NSString stringWithFormat:@"%@",[[self dict] objectForKey:@"nombre_completo"]] ;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ %@, %@", [[self dict] objectForKey:@"tipovia"], [[self dict] objectForKey:@"via"], [[self dict] objectForKey:@"num_via"]];
        
        return cell;
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en MedicosController.tableView.cellForRowAtIndexPath: %@", e);
    }
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    @try {
        [self setDict:[[self rows] objectAtIndex:indexPath.row]];
        
        [self setDestination:[NSString stringWithFormat:@"%@", [[self dict] objectForKey:@"domicilio_gps"]]];
        [self setFullName:[NSString stringWithFormat:@"%@", [[self dict] objectForKey:@"nombre_completo"]]];
        [self setAddress:[NSString stringWithFormat:@"%@ %@, %@", [[self dict] objectForKey:@"tipovia"], [[self dict] objectForKey:@"via"], [[self dict] objectForKey:@"num_via"]]];
        [self setPhone:[NSString stringWithFormat:@"%@", [[self dict] objectForKey:@"telefono"]]];
        [self setEspecDescription:[NSString stringWithFormat:@"%@", [[self dict] objectForKey:@"especialidad"]]];
        
        [self performSegueWithIdentifier:[NSString stringWithFormat:@"%s", "MostrarMapa"] sender:self];
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en MedicosController.tableView.didSelectRowAtIndexPath: %@", e);
    }
}

-(void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender{
    @try {
        if ([[segue identifier] isEqualToString:@"MostrarMapa"]) {
            GPSController *nextView = [segue destinationViewController];
            
            [nextView setFullName:[self fullName]];
            [nextView setDestination:[self destination]];
            [nextView setAddress:[self address]];
            [nextView setPhone:[self phone]];
            [nextView setEspecDescription:[self especDescription]];
            
            [nextView setState:[self state]];
            [nextView setCity:[self city]];
            [nextView setEspec:[self espec]];
            
            [nextView setUserCode:[self userCode]];
            [nextView setUserType:[self userType]];
        } else if ([[segue identifier] isEqualToString:@"BackFromMed"]) {
            BuscarMedicoController *nextView = [segue destinationViewController];
            
            [nextView setUserCode:_userCode];
            [nextView setUserType:_userType];
        }
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en MedicosController.prepareForSegue: %@", e);
    }
}

- (void)alertView:(UIAlertView *)alerta clickedButtonAtIndex:(NSInteger)buttonIndex
{
    @try {
        NSString *title = [alerta buttonTitleAtIndex:buttonIndex];
        
        if([title isEqualToString:@"SI"])
        {
            [self performSegueWithIdentifier:[NSString stringWithFormat:@"%s", "CerrarSesion"] sender:self];
            
        }
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en MedicosController.alertView.clickedButtonAtIndex: %@", e);
    }
}

- (IBAction)exitButtonClick:(id)sender {
    @try {
        UIAlertView *alert = [[UIAlertView alloc]
                              initWithTitle:[NSString stringWithFormat:@"Atención:"]
                              message:[NSString stringWithFormat:@"¿Seguro que quiere abandonar la sesión?."]
                              delegate:self
                              cancelButtonTitle:@"SI"
                              otherButtonTitles:@"NO", nil];
        [alert show];
    }
    @catch (NSException *e) {
        NSLog(@"Excepción en MedicosController.exitButtonClick: %@", e);
    }
}

@end
